# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-11-27 20:43
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Receipt',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('internal_uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Дата создания чека')),
                ('initiated_at', models.DateTimeField(blank=True, null=True, verbose_name='Дата инициализации чека в системе оператора')),
                ('received_at', models.DateTimeField(blank=True, null=True, verbose_name='Дата получения чека от оператора')),
                ('failed_at', models.DateTimeField(blank=True, null=True, verbose_name='Дата ошибки')),
                ('status', models.CharField(choices=[('created', 'Ожидает инициации в системе оператора'), ('initiated', 'Иницирован в системе оператора'), ('received', 'Получен от оператора'), ('no_email_phone', 'Отсутствует email/phone'), ('failed', 'Ошибка')], default='created', max_length=16, verbose_name='Статус чека')),
                ('uuid', models.TextField(editable=False, help_text='Идентификатор чека платежа в системе оператора', null=True, verbose_name='Идентификатор чека')),
                ('content', django.contrib.postgres.fields.jsonb.JSONField(editable=False, null=True, verbose_name='Содержимое чека')),
                ('user_email', models.CharField(max_length=254, null=True, verbose_name='Email пользователя')),
                ('user_phone', models.CharField(max_length=32, null=True, verbose_name='Телефон пользователя')),
                ('purchase_price', models.DecimalField(decimal_places=2, max_digits=8, null=True, verbose_name='Цена покупки')),
                ('purchase_name', models.TextField(null=True, verbose_name='Наименование покупки')),
            ],
            options={
                'verbose_name': 'Чек Атола',
                'verbose_name_plural': 'Чеки Атола',
                'ordering': ['id'],
            },
        ),
    ]
